#! /usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const list_files_js_1 = require("../modules/helpers/list-files.js");
const get_arg_js_1 = require("../parsers/get-arg.js");
const files_js_1 = require("../configs/files.js");
const format_js_1 = require("../services/format.js");
const kill_js_1 = require("../modules/helpers/kill.js");
const env_js_1 = require("../modules/helpers/env.js");
const poku_js_1 = require("../modules/essentials/poku.js");
const write_js_1 = require("../services/write.js");
const options_js_1 = require("../parsers/options.js");
(async () => {
    if ((0, get_arg_js_1.hasArg)('version') || (0, get_arg_js_1.hasArg)('v', '-')) {
        const { VERSION } = require('../configs/poku.js');
        write_js_1.Write.log(VERSION);
        return;
    }
    if ((0, get_arg_js_1.hasArg)('help') || (0, get_arg_js_1.hasArg)('h', '-')) {
        const { help } = require('./help.js');
        help();
        return;
    }
    const enforce = (0, get_arg_js_1.hasArg)('enforce') || (0, get_arg_js_1.hasArg)('x', '-');
    const configFile = (0, get_arg_js_1.getArg)('config') || (0, get_arg_js_1.getArg)('c', '-');
    const defaultConfigs = await (0, options_js_1.getConfigs)(configFile);
    const dirs = (0, get_arg_js_1.getPaths)('-') ??
        (defaultConfigs?.include
            ? Array.prototype.concat(defaultConfigs?.include)
            : ['.']);
    const filter = (0, get_arg_js_1.getArg)('filter') ?? defaultConfigs?.filter;
    const exclude = (0, get_arg_js_1.getArg)('exclude') ?? defaultConfigs?.exclude;
    const killPort = (0, get_arg_js_1.getArg)('killPort');
    const killRange = (0, get_arg_js_1.getArg)('killRange');
    const killPID = (0, get_arg_js_1.getArg)('killPid');
    
    const denoAllow = (0, get_arg_js_1.argToArray)('denoAllow') ?? defaultConfigs?.deno?.allow;
    const denoDeny = (0, get_arg_js_1.argToArray)('denoDeny') ?? defaultConfigs?.deno?.deny;
    const denoCJS = (0, get_arg_js_1.getArg)('denoCjs')
        ?.split(',')
        .map((a) => a.trim())
        .filter((a) => a) ||
        (0, get_arg_js_1.hasArg)('denoCjs') ||
        defaultConfigs?.deno?.cjs;
    
    const quiet = (0, get_arg_js_1.hasArg)('quiet') || (0, get_arg_js_1.hasArg)('q', '-') || defaultConfigs?.quiet;
    const debug = (0, get_arg_js_1.hasArg)('debug') || (0, get_arg_js_1.hasArg)('d', '-') || defaultConfigs?.debug;
    const failFast = (0, get_arg_js_1.hasArg)('failFast') || defaultConfigs?.failFast;
    const watchMode = (0, get_arg_js_1.hasArg)('watch') || (0, get_arg_js_1.hasArg)('w', '-');
    const hasEnvFile = (0, get_arg_js_1.hasArg)('envFile');
    const concurrency = (() => {
        const value = Number((0, get_arg_js_1.getArg)('concurrency'));
        return Number.isNaN(value) ? defaultConfigs?.concurrency : value;
    })();
    const sequential = (0, get_arg_js_1.hasArg)('sequential');
    if (dirs.length === 1)
        files_js_1.states.isSinglePath = true;
    if ((0, get_arg_js_1.hasArg)('listFiles')) {
        const { listFiles } = require('../modules/helpers/list-files.js');
        const files = [];
        write_js_1.Write.hr();
        for (const dir of dirs)
            files.push(...(await listFiles(dir, {
                filter: typeof filter === 'string'
                    ? new RegExp((0, list_files_js_1.escapeRegExp)(filter))
                    : filter,
                exclude: typeof exclude === 'string'
                    ? new RegExp((0, list_files_js_1.escapeRegExp)(exclude))
                    : exclude,
            })));
        write_js_1.Write.log(files
            .sort()
            .map((file) => `${(0, format_js_1.format)('-').dim()} ${file}`)
            .join('\n'));
        write_js_1.Write.hr();
        write_js_1.Write.log(`Total test files: ${(0, format_js_1.format)(String(files.length)).bold()}`);
        write_js_1.Write.hr();
        return;
    }
    if (enforce) {
        const { checkFlags } = require('./enforce.js');
        checkFlags();
    }
    const tasks = [];
    
    if (killPort || defaultConfigs?.kill?.port) {
        const ports = killPort?.split(',').map(Number) || defaultConfigs?.kill?.port || [];
        tasks.push(kill_js_1.kill.port(ports));
    }
    if (killRange || defaultConfigs?.kill?.range) {
        const ranges = killRange?.split(',') ||
            defaultConfigs?.kill?.range?.map((range) => `${range[0]}-${range[1]}`) ||
            [];
        for (const range of ranges) {
            const ports = range.split('-').map(Number);
            const startsAt = ports[0];
            const endsAt = ports[1];
            tasks.push(kill_js_1.kill.range(startsAt, endsAt));
        }
    }
    if (killPID || defaultConfigs?.kill?.pid) {
        const PIDs = killPID?.split(',').map(Number) || defaultConfigs?.kill?.pid || [];
        tasks.push(kill_js_1.kill.pid(PIDs));
    }
    
    if (hasEnvFile || defaultConfigs?.envFile) {
        const envFilePath = (0, get_arg_js_1.getArg)('envFile') ?? defaultConfigs?.envFile;
        tasks.push((0, env_js_1.envFile)(envFilePath));
    }
    const options = {
        filter: typeof filter === 'string' ? new RegExp((0, list_files_js_1.escapeRegExp)(filter)) : filter,
        exclude: typeof exclude === 'string' ? new RegExp((0, list_files_js_1.escapeRegExp)(exclude)) : exclude,
        concurrency,
        sequential,
        quiet,
        debug,
        failFast,
        deno: {
            allow: denoAllow,
            deny: denoDeny,
            cjs: denoCJS,
        },
        noExit: watchMode,
        beforeEach: 'beforeEach' in defaultConfigs ? defaultConfigs.beforeEach : undefined,
        afterEach: 'afterEach' in defaultConfigs ? defaultConfigs.afterEach : undefined,
    };
    if (debug || defaultConfigs?.debug) {
        write_js_1.Write.hr();
        write_js_1.Write.log(`${(0, format_js_1.format)(' Debug Enabled ').bg('brightBlue')}\n`);
        write_js_1.Write.log(`${(0, format_js_1.format)('…').info().italic()} ${(0, format_js_1.format)('Paths').bold()}`);
        console.table(dirs);
        write_js_1.Write.log('\n');
        write_js_1.Write.log(`${(0, format_js_1.format)('…').info().italic()} ${(0, format_js_1.format)('Options').bold()}`);
        console.dir(options, { depth: null, colors: true });
    }
    await Promise.all(tasks);
    await (0, poku_js_1.poku)(dirs, options);
    if (watchMode) {
        const { startWatch } = require('./watch.js');
        await startWatch(dirs, options);
    }
})();
